json = require ('lib/json')
local h, w
local squares
local sqrSz
local currColor
local colors
local darkColors = {lineColorLight = {1,1,1,0.6}, lineColor = {1,1,1,1}, lineColorBold = {0.5,0.5,1,1}, background = {0,0,0,1}}
local lightColors = {lineColorLight = {0,0,0,0.6}, lineColor = {0,0,0,1}, lineColorBold = {0.5,0.5,1,1}, background = {1,1,1,1}}
local xoff
local yoff
done = false

function love.load()
    love.window.setTitle("Stitchin' Time")
    squares = {}
    loadSettings()
end

function loadSettings()
    local settings = json.decode( love.filesystem.read('settings.json'))
    if settings.ColorScheme == 'dark' then
        colorScheme = darkColors
    elseif settings.ColorScheme == 'light' then
        colorScheme = lightColors
    else
        colorScheme = darkColors
    end
    if settings.LoadFile then
        loadSavedFile(json.decode( love.filesystem.read(settings.LoadFile)))
    end
    love.graphics.setBackgroundColor(colorScheme.background)
    h, w = settings.hSquares, settings.vSquares
    setSqrSz()
    colors = settings.colors
    currColor = 1
end

function save()
    success, message = love.filesystem.write( os.time() .. '.json', json.encode({squaresStringTable(), h, w}))
end

function squaresStringTable()
    local stringTable = {}

    for i in pairs(squares) do
        for j in pairs(squares[i]) do
            if not stringTable[tostring(i)] then
                stringTable[tostring(i)] = {}
            end
            stringTable[tostring(i)][tostring(j)] = squares[i][j]
        end
    end
    return stringTable
end

function loadSavedFile(savedFile)
    squares = {}
    h = savedFile[2]
    w = savedFile[3]
    for i in pairs(savedFile[1]) do
        for j in pairs(savedFile[1][i]) do
            if not squares[tonumber(i)] then
                squares[tonumber(i)] = {}
            end
            squares[tonumber(i)][tonumber(j)] = savedFile[1][i][j]
        end
    end
end

function twoSixFiveToDecimal(rgba)
    return {rgba[1]/256, rgba[2]/256, rgba[3]/256, (rgba[4] or 256)/256}
end

function love.resize()
    setSqrSz()
end

function setSqrSz()
    sqrSz = math.min((love.graphics.getWidth())/(w + 5), (love.graphics.getHeight())/(h+2))
    xoff = sqrSz * 3
    yoff = sqrSz
end

function love.update(dt)
    if love.mouse.isDown(1) then
        addSquare(toGrid2(love.mouse.getPosition()))
    elseif love.mouse.isDown(2) then
        removeSquare(toGrid2(love.mouse.getPosition()))
    end
end

function love.draw()
    drawSquares()
    drawGraph()
    drawColors()
end

function addSquare(x, y)
    if x < w and x > 0 and y < h and y > 0 then
        if not squares[x] then
            squares[x] = {}
        end
        squares[x][y] = colors[currColor]
    end
end

function removeSquare(x,y)
    if squares[x] then
        squares[x][y] = nil
    end
end

function drawSquares()
    local x, y
    for xpos in pairs(squares) do
        for ypos in pairs(squares[xpos]) do
            x, y = toWorld2(xpos, ypos)
            love.graphics.setColor(twoSixFiveToDecimal(squares[xpos][ypos].rgb))
            love.graphics.rectangle('fill', x, y, sqrSz, sqrSz)
            love.graphics.setColor(colorScheme.background)
            love.graphics.printf(squares[xpos][ypos].symbol, x, y, sqrSz+2, 'center')
        end
    end
end

function drawGraph()
    love.graphics.setColor(colorScheme.lineColorLight) -- every 1 across
    for i = 1, h+1 do
        love.graphics.line(toWorld(1,i,w+1,i))
    end
    love.graphics.setColor(colorScheme.lineColorBold) -- every 10 across
    for i = 11, h, 10 do
        love.graphics.line(toWorld(1,i,w+1,i))
    end

    love.graphics.setColor(colorScheme.lineColorLight) -- every 1 down
    for i = 1, w+1 do
        love.graphics.line(toWorld(i, 1,i, h+1)) 
    end
    love.graphics.setColor(colorScheme.lineColorBold) -- every 10 down
    for i = 11, w, 10 do
        love.graphics.line(toWorld(i, 1,i, h))
    end
end

function drawColors()
    local count = 0
    for i in pairs(colors) do
        count = count + 1
        love.graphics.setColor(twoSixFiveToDecimal(colors[i].rgb))
        love.graphics.rectangle('fill', 10, 10 + count*sqrSz*2, sqrSz, sqrSz)
        love.graphics.setColor(colorScheme.background)
        love.graphics.printf(colors[i].symbol, 10, 10 + count*sqrSz*2, sqrSz+2, 'center')
    end
    love.graphics.setColor(colorScheme.lineColor)
    love.graphics.rectangle('line', 8, 8 + currColor*sqrSz*2,sqrSz+4,sqrSz+4)
end

function toWorld(x1,y1,x2,y2)
    local r1, r2 = toWorld2(x1, y1)
    local r3, r4 = toWorld2(x2, y2)
    return r1, r2, r3, r4
end
function toWorld2(x, y)
    return (x)*sqrSz + xoff, (y)*sqrSz + yoff
end

function toGrid(x1,y1,x2,y2)
    local r1, r2 = toGrid2(x1, y1)
    local r3, r4 = toGrid2(x2, y2)
    return r1, r2, r3, r4
end

function toGrid2(x,y)
    return round((x - xoff - sqrSz/2)/sqrSz), round((y - yoff - sqrSz/2)/sqrSz)
end

function round(num, numDecimalPlaces)
    local mult = 10^(numDecimalPlaces or 0)
    return math.floor(num * mult + 0.5) / mult
  end

function takeScreenShot()
    love.graphics.captureScreenshot(os.time() .. '.png') -- saves deep in the filesystem. How to save locally? heck if I know
end

function love.keypressed(k)
    if k == 'p' then
        takeScreenShot()
    elseif k == 'down' then
        nextColor()
    elseif k == 'up' then
        prevColor()
    elseif k == 's' then
        save()
    end
end

function love.wheelmoved(x,y)
    if y > 0 then
        prevColor()
    elseif y < 0 then
        nextColor()
    end
end

function nextColor()
    currColor = (currColor % #colors) + 1
end

function prevColor()
    currColor = currColor - 1
    if currColor == 0 then currColor = #colors end
end

function once(testString)
    if not done then
        print('hey', testString)
        done = true
    end
end
